@@ -141,6 +141,13 @@ class TestRabbitTransportURL(test_utils.BaseTestCase):
         ('rabbit',
          dict(url='kombu+rabbit://user:password@host:10/virtual_host',
               expected=['amqp://user:password@host:10/virtual_host'])),
+        ('rabbit_ipv6',
+         dict(url='kombu+rabbit://u:p@[fd00:beef:dead:55::133]:10/vhost',
+              skip_py26='python 2.6 has broken urlparse for ipv6',
+              expected=['amqp://u:p@[fd00:beef:dead:55::133]:10/vhost'])),
+        ('rabbit_ipv4',
+         dict(url='kombu+rabbit://user:password@10.20.30.40:10/vhost',
+              expected=['amqp://user:password@10.20.30.40:10/vhost'])),
     ]
 
     def setUp(self):
@@ -152,6 +159,9 @@ class TestRabbitTransportURL(test_utils.BaseTestCase):
     @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection.ensure')
     @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection.reset')
     def test_transport_url(self, fake_ensure_connection, fake_reset):
+        if hasattr(self, 'skip_py26') and sys.version_info < (2, 7):
+            self.skipTest(self.skip_py26)
+
         transport = messaging.get_transport(self.conf, self.url)
         self.addCleanup(transport.cleanup)
         driver = transport._driver
